Package: proxy
Type: Package
Title: Distance and Similarity Measures
Version: 0.4-27
Authors@R: c(person(given = "David", family = "Meyer", role = c("aut", "cre"), email = "David.Meyer@R-project.org"),
             person(given = "Christian", family = "Buchta", role = "aut"))
Description: Provides an extensible framework for the efficient calculation of auto- and cross-proximities, along with implementations of the most popular ones. 
Depends: R (>= 3.4.0)
Imports: stats, utils
Suggests: cba
Collate: registry.R database.R dist.R similarities.R dissimilarities.R
        util.R seal.R
License: GPL-2
NeedsCompilation: yes
Packaged: 2022-06-08 21:36:02 UTC; meyer
Author: David Meyer [aut, cre],
  Christian Buchta [aut]
Maintainer: David Meyer <David.Meyer@R-project.org>
Repository: CRAN
Date/Publication: 2022-06-09 06:15:32 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 02:15:09 UTC; unix
