Package: proxy
Type: Package
Title: Distance and Similarity Measures
Version: 0.4-29
Authors@R: c(person(given = "David", family = "Meyer", role = c("aut", "cre"),
                    email = "David.Meyer@R-project.org",
		    comment = c(ORCID = "0000-0002-5196-3048")),	    
             person(given = "Christian", family = "Buchta", role = "aut"))
Description: Provides an extensible framework for the efficient calculation of auto- and cross-proximities, along with implementations of the most popular ones. 
Depends: R (>= 3.4.0)
Imports: stats, utils
Suggests: cba
Collate: registry.R database.R dist.R similarities.R dissimilarities.R
        util.R seal.R
License: GPL-2 | GPL-3
NeedsCompilation: yes
Packaged: 2025-12-29 16:26:15 UTC; hornik
Author: David Meyer [aut, cre] (ORCID: <https://orcid.org/0000-0002-5196-3048>),
  Christian Buchta [aut]
Maintainer: David Meyer <David.Meyer@R-project.org>
Repository: CRAN
Date/Publication: 2025-12-29 16:42:24 UTC
